import socket

# Server echo: accetta la connessione da un client alla volta.
# Legge una stringa mandata dal client (fino a 2048 byte) e la 
# rimanda indietro tale e quale.
#
class EchoServer():

	def __init__(self, address, port):

		self.address_tuple = (address, port)		
	
		# AF_INET: uso IPv4
		# SOCK_STREAM: uso TCP (protocollo trasporto affidabile e orientato alla connessione)
		tcpsock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		# Quando la connessione viene chiusa, il socket viene lasciato in uno stato
		# di TIME_WAIT e non puo` essere utilizzato di nuovo immediatamente.
		# Il messaggio di errore e`:
		# Address already in use
		# Si puo` dire al server di riutilizzarlo pero`:
		#
		tcpsock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
		#
		tcpsock.bind(self.address_tuple)
		
		print("server: in ascolto su: ", tcpsock.getsockname());
		# 0 massimo numero di connessioni _in_ _coda_ 
		#
		tcpsock.listen(0)

		try:
			# Il server rimane in attesa di nuovi client fino a quando viene terminato 
			# con la pressione simultanea dei tasti Control-C
			while True:
				# aspetto una nuova connessione
				# clientsock: socket utilizzabile per ricevere e mandare dati
				# cliaddr:    l'indirizzo del client associato al socket
				(clientsock, cliaddr) = tcpsock.accept()
				# ricevo dati tipo 'byte'
				data = clientsock.recv(2048)
				# li rimando al client
				clientsock.sendall(data)				
				# chiudo il socket dopo aver fatto echo
				clientsock.close()
				# aspetto una nuova connessione da un altro client
				
		except KeyboardInterrupt: # intercetto l'interruzione da tastiera
			print("premuto Control-C: esco!\n")
			tcpsock.close()	
			

# Chiede una stringa dall'utente. La invia al server e aspetta una risposta
#
class EchoClient:
	def __init__(self, servaddr, servport):
		
		clisocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		clisocket.connect((servaddr, servport))
		
		s = input("\nclient: inserisci la stringa da mandare al server: ")
		# da string in bytes
		data = bytes(s, 'utf-8')
		# mando al server
		clisocket.sendall(data)
		# aspetto la risposta
		data = clisocket.recv(2048)
		#
		# stampo la risposta dall'echo server
		# da byte a string mediante la funzione decode() dell'oggetto byte
		# utf-8 e` l'encoding
		print("ricevuto: \"" + data.decode('utf-8') + "\"\n")
		clisocket.close()
