import threading
import time
import sys

# Due esempi di thread molto semplici.
# Entrambi fanno la stessa cosa: il thread principale (Esempio e EsempioSincronizzato, rispettivamente),
# ogni 500ms aggiunge un intero in coda a una lista (mediante append() )
# Il secondo thread (EsempioThread e EsempioThreadSincronizzato) ogni 500ms rimuove dalla lista 
# tutti gli n - 1 elementi lasciando solo l'ultimo (mediante pop(0) )
#
# La versione sincronizzata protegge i dati in comune con un lock
#
# Documentazione thread:
#
# http://docs.python.org/3/library/threading.html
#
#
# Uso:
# from threads import * (o F5 su idle, al solito)
# 
# Esempio() per lanciare il primo esempio
#
# EsempioSincronizzato() per lanciare il secondo
#

class EsempioThread(threading.Thread):
	def __init__(self, alist):
		threading.Thread.__init__(self)
		self.__interrompi = False
		self.alist = alist
	
	# da implementare per forza quando si eredita da threading.Thread
	# viene invocata automaticamente dal sistema dei thread in seguito
	# alla chiamata start() dentro un altro thread
	def run(self):
		n = 0
		while n < 1000:
			while len(self.alist) > 1:
				self.alist.pop(0)
			print("list ", self.alist, self)
			time.sleep(0.5)
			
			if self.__interrompi:
				break		
	def esci(self):
		self.__interrompi = True

class Esempio:
	
	def __init__(self):
		alist = [0]
		# Creo il thread
		esth = EsempioThread(alist)
		# lo faccio partire
		esth.start()
		
		# l'esecuzione continua in (questo) thread principale
		n = 0
		while n < 100:
			n = n + 1
			alist.append(n)
			print("\033[1;32mlist ", alist, "\033[0m", self)
			time.sleep(0.5)
			
		esth.esci()
		print("aspetto la terminazione del thread...")				
		# aspetto il thread 
		esth.join()
		print("ok, thread uscito");			
	

# Versione in cui i dati sono protetti da accesso concorrente mediante threading.RLock
#
class EsempioThreadSincronizzato(threading.Thread):
	def __init__(self, alist):
		threading.Thread.__init__(self)
		self.__interrompi = False
		self.alist = alist
		
	def run(self):
		for i in range(0, 1000):
			# lock variabile di classe: serve anteporre 'EsempioSincronizzato.'
			EsempioSincronizzato.lock.acquire()
			while len(self.alist) > 1:
				self.alist.pop(0)
			print("list ", self.alist, self)
			EsempioSincronizzato.lock.release()
			
			time.sleep(0.5)
			
			if self.__interrompi:
				break
		
	def esci(self):
		self.__interrompi = True

class EsempioSincronizzato:
	
	
	# semaforo per la gestione concorrente delle risorse condivise tra i due thread
	# lock e` una variabile di classe e non di istanza (le variabili di istanza 
	# sono precedute da self.nome_variabile)
	# Va pertanto richiamata con EsempioSincronizzato.lock, sia dai metodi della
	# classe stessa che da EsempioThreadSincronizzato
	# 
	lock = threading.RLock()
	
	def __init__(self):
		
		
		alist = [0]
		esth = EsempioThreadSincronizzato(alist)
		esth.start()
		
		n = 0
		while n < 100:
			# lock variabile di classe: serve anteporre 'EsempioSincronizzato.'
			n = n + 1
			EsempioSincronizzato.lock.acquire()
			alist.append(n)
			print("\033[1;32mlist ", alist, "\033[0m", self)
			EsempioSincronizzato.lock.release()
			
			time.sleep(0.5)
				
		esth.esci()
		print("aspetto la terminazione del thread...", end="")				
		# aspetto il thread 
		esth.join()
		print("ok, thread uscito");
